#include <IGPUProgrammingServices.h>
#include <IMaterialRendererServices.h>
#include "IPostProcessingHandler.hpp"
#include "CScreenQuadSceneNode.hpp"
#include "CScreenQuad.hpp"

//  Shaders list:
#include "CBlurShader.hpp"
#include "C2dShock.hpp"
#include "CDepthMapCallback.hpp"


// TODO : remove, use just for debug
#include <fusion/fusion.hpp>

using namespace irr;
using namespace video;
using namespace scene;



IPostProcessingHandler::IPostProcessingHandler() :
//    m_supported(false)
//    , m_render_target(nullptr)
VertexProfile(EVST_COUNT)
,PixelProfile(EPST_COUNT)
, ScreenQuad(0)
,ScreenRTT(0)
,BlurCB(0)
,DepthMapCB(0)
,BlurHP(0)
,BlurVP(0)
{
//    m_boost_amount = 0.0f;
    TempRTT = 0;
//    TempRTT[0] = 0;
//    TempRTT[1] = 0;
}


IPostProcessingHandler::~IPostProcessingHandler()
{

}


//EVDF_TEXTURE_NPOT


//Nausicaa of The Valley of The Wind [Blu-Ray 720p].part1.rar

/** Initialization */
// TODO ? passer le log level
bool IPostProcessingHandler::init(video::IVideoDriver* video_driver, core::stringc& msg)
{

    bool vertProfileSet = false;
    bool pixProfileSet = false;



    //! Check GLSL support
    if( video_driver->queryFeature(video::EVDF_ARB_GLSL) ){
        msg.append("GLSL supported.\n");
    }
    else {
        msg.append( "GLSL not supported. Aborting...\n");
        return false;
    }

    //! Check RTT support
    if( video_driver->queryFeature(video::EVDF_RENDER_TO_TARGET) ){
        msg.append("Render To Target supported.\n");
    }
    else {
        msg.append( "Render to target not supported. Aborting...\n");
        return false;
    }


    //! Compute highest vertex shader version supported
/*
    for(irr::u32 i(EVDF_VERTEX_SHADER_1_1); i < EVDF_VERTEX_SHADER_3_0 ; ++i)
    {
        const E_VERTEX_SHADER_TYPE vertProfile = static_cast<E_VERTEX_SHADER_TYPE>(i);
        //video::EVDF_ARB_FRAGMENT_PROGRAM_1
        if ( video_driver->queryFeature( vertProfile  ){
            VertexProfile = vertProfile ;
            vertProfileSet = true;
        }
        //
    }

    if(vertProfile != EVST_COUNT){
        msg.append("Vertex Profile set. Highest supported is :");
        msg.append( VERTEX_SHADER_TYPE_NAMES[VertexProfile] );
        msg.append( "\n" );
    }
    else {
        msg.append( "Vertex shaders not supported" );
        return false;
    }
*/
    VertexProfile = video_driver->queryFeature( video::EVDF_VERTEX_SHADER_3_0  ) ? EVST_VS_3_0 : EVST_VS_2_0;


    //! Compute highest pixel shader version supported
    /*
    for(irr::u32 i = 0; i < EPST_COUNT; ++i)
    {
        const E_PIXEL_SHADER_TYPE pixProfile = static_cast<E_PIXEL_SHADER_TYPE>(i);
        if ( video_driver->queryFeature( pixProfile  ){
            PixelProfile = pixProfile ;
//            pixProfileSet  = true;
        }
    }


    if(pixProfile != EPST_COUNT){
        msg.append("Pixel Profile set. Highest supported is :");
        msg.append( PIXEL_SHADER_TYPE_NAMES[PixelProfile] );
    }
    else{
        msg.append( "Pixel shaders not supported" );
        return false;
    }
*/
    PixelProfile = video_driver->queryFeature( video::EVDF_PIXEL_SHADER_3_0  ) ? video::EPST_PS_3_0 : video::EPST_PS_2_0;

    BlurCB = new CBlurShader();

    //! Create Screen quad ?
    //! TODO might use
    Driver = video_driver;

    video::IGPUProgrammingServices* gpu = Driver->getGPUProgrammingServices();

    BlurHP  = gpu->addHighLevelShaderMaterialFromFiles(
                    "shaders/default.vert.glsl", "main", VertexProfile,
                    "shaders/BlurHP.perso.glsl", "main", PixelProfile
                    , BlurCB
                    , video::EMT_SOLID
                    );

    BlurVP  = gpu->addHighLevelShaderMaterialFromFiles(
                    "shaders/default.vert.glsl", "main", VertexProfile,
                    "shaders/BlurVP.perso.glsl", "main", PixelProfile
                    , BlurCB
                    , video::EMT_SOLID
                    );


    ScreenRTT = Driver->addRenderTargetTexture( Driver->getScreenSize() , "Screen RTT");
    TempRTT = Driver->addRenderTargetTexture( Driver->getScreenSize() , "TempRTT0");
//    TempRTT[0] = Driver->addRenderTargetTexture( Driver->getScreenSize() , "TempRTT0");
//    TempRTT[1] = Driver->addRenderTargetTexture( Driver->getScreenSize() , "TempRTT1");
    return true;
}



void
IPostProcessingHandler::blurTexture(irr::video::ITexture* in,irr::video::ITexture* out)
{
    //!
    //Driver->

    ///////////////////////////////////////
    ////    HORIZONTAL BLUR
	///////////////////////////////////////
    Driver->setRenderTarget(TempRTT, true, true,  video::SColor(0,0,0,255));

//            node->getMaterial(0).MaterialType = EMT_SOLID;
    Quad.getMaterial().MaterialType = (E_MATERIAL_TYPE)BlurHP;
    Quad.getMaterial().Lighting = false;
    Quad.getMaterial().setTexture(0,in);
    Quad.render( Driver );




    ///////////////////////////////////////
    ////    VERTICAL BLUR
	///////////////////////////////////////
	// TODO remplacer par des false
    Driver->setRenderTarget(out, true, true,  video::SColor(0,0,0,255));
    Quad.getMaterial().MaterialType = (E_MATERIAL_TYPE)BlurVP;
    Quad.getMaterial().Lighting = false;
    Quad.getMaterial().setTexture(0, TempRTT);
    Quad.render( Driver );

//    latestRender = rtAfterVBlur;
}



void IPostProcessingHandler::render(video::ITexture* texture, const core::position2di& pos )
{

//    if( texture == 0)   return;
//
//    Quad.getMaterial().setTexture(0,texture);
//    Quad.render( Driver );


    core::dimension2du currentResolution;
//    currentResolution = Driver->getScreenSize();
    currentResolution = texture->getSize();

//    currentResolution.Width = 256;
//    currentResolution.Height = 256;

//    _INFO << " texture path" << texture->getName().getPath().c_str();
//    _INFO << " texture internal name " << texture->getName().getInternalName().c_str();
//    _INFO << " is a RTT ?" << texture->isRenderTarget () ;

    core::rect<s32> rectPos = core::rect<s32>( pos, core::dimension2di(currentResolution.Width,currentResolution.Height) );
    core::rect<s32> dims = core::rect<s32>( 0,0, currentResolution.Width,currentResolution.Height );

    Driver->draw2DImage( texture, rectPos, dims );

//    Driver->draw2DImage( texture, core::position2di(0,0) );


}







irr::s32
IPostProcessingHandler::addShaderMaterial(const irr::core::stringc& vertShader,const irr::core::stringc& fragShader)
{
//gpu->addHighLevelShaderMaterialFromFiles(
//                    ("shaders/motion_blur.vert").c_str(), "main", video::EVST_VS_2_0,
//                    ("shaders/lifebar.glsl.frag").c_str(), "main", video::EPST_PS_2_0,
//                    this, video::EMT_SOLID);
    return 0;
}























#if 0

/** Initialization */
void PostProcessing::init(video::IVideoDriver* video_driver)
{
    // Check if post-processing is supported on this hardware
    m_supported = false;
    if( video_driver->queryFeature(video::EVDF_ARB_GLSL) &&
        video_driver->queryFeature(video::EVDF_PIXEL_SHADER_2_0) &&
        video_driver->queryFeature(video::EVDF_RENDER_TO_TARGET))
    {
        m_supported = true;
    }

    // Initialization
    if(m_supported)
    {
        // Render target
        m_render_target = video_driver->addRenderTargetTexture(video_driver->getScreenSize(), "postprocess");
        if(!m_render_target)
        {
            fprintf(stderr, "Couldn't create the render target for post-processing, disabling it\n");
            UserConfigParams::m_postprocess_enabled = false;
        }

        // Material and shaders
        IGPUProgrammingServices* gpu = video_driver->getGPUProgrammingServices();
        s32 material_type = gpu->addHighLevelShaderMaterialFromFiles(
                    (file_manager->getDataDir() + "shaders/motion_blur.vert").c_str(), "main", video::EVST_VS_2_0,
                    (file_manager->getDataDir() + "shaders/motion_blur.frag").c_str(), "main", video::EPST_PS_2_0,
                    this, video::EMT_SOLID);
        m_material.MaterialType = (E_MATERIAL_TYPE)material_type;
        m_material.setTexture(0, m_render_target);
        m_material.Wireframe = false;
        m_material.Lighting = false;
        m_material.ZWriteEnable = false;
    }

	Mesh = new irr::scene::CMeshBuffer<irr::video::S3DVertex>( );
	Mesh->Vertices.reallocate( 4 );
	Mesh->Indices.reallocate( 6 );

	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f, -1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 0.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex( -1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 0.0f, 1.0f ) );
	Mesh->Vertices.push_back( irr::video::S3DVertex(  1.0f,  1.0f, 0.0f, 0.0f, 0.0f, 0.0f, irr::video::SColor( 255u, 255u, 255u, 255u ), 1.0f, 1.0f ) );

	Mesh->Indices.push_back( 0u );
	Mesh->Indices.push_back( 2u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 3u );
	Mesh->Indices.push_back( 1u );
	Mesh->Indices.push_back( 2u );


}



/** Setup the render target */
void PostProcessing::beginCapture()
{
    if(!m_supported || !UserConfigParams::m_postprocess_enabled ||
       race_manager->getNumPlayers() > 1)
        return;

    // don't capture the input when we have no post-processing to add
    // it will be faster and this ay we won't lose anti-aliasing
    if (m_boost_amount <= 0.0f)
    {
        m_used_pp_this_frame = false;
        return;
    }

    m_used_pp_this_frame = true;
    irr_driver->getVideoDriver()->setRenderTarget(m_render_target, true, true);
}


/** Render the post-processed scene */
void PostProcessing::render()
{
//    if(!m_supported || !UserConfigParams::m_postprocess_enabled ||
//       race_manager->getNumPlayers() > 1)
//        return;
//
//    if (!m_used_pp_this_frame)
//    {
//        return;
//    }

    // Draw the fullscreen quad while applying the corresponding post-processing shaders
    video::IVideoDriver*    video_driver = irr_driver->getVideoDriver();
    /*video::S3DVertex        vertices[6];

    video::SColor white(0xFF, 0xFF, 0xFF, 0xFF);
    vertices[0] = irr::video::S3DVertex(-1.0f,-1.0f,0.0f,0,0,1, white, 0.0f,1.0f);
    vertices[1] = irr::video::S3DVertex(-1.0f, 1.0f,0.0f,0,0,1, white, 0.0f,0.0f);
    vertices[2] = irr::video::S3DVertex( 1.0f, 1.0f,0.0f,0,0,1, white, 1.0f,0.0f);
    vertices[3] = irr::video::S3DVertex( 1.0f,-1.0f,0.0f,0,0,1, white, 1.0f,1.0f);
    vertices[4] = irr::video::S3DVertex(-1.0f,-1.0f,0.0f,0,0,1, white, 0.0f,1.0f);
    vertices[5] = irr::video::S3DVertex( 1.0f, 1.0f,0.0f,0,0,1, white, 1.0f,0.0f);

    u16 indices[6] = {0, 1, 2, 3, 4, 5};

    video_driver->setMaterial(m_material);
    video_driver->drawIndexedTriangleList(&vertices[0], 6, &indices[0], 2);
    */

		driver->setRenderTarget( target, false, false );
		driver->setMaterial( Tmats[ i ] );
		driver->drawMeshBuffer( Mesh );
}




#endif
