/**
\file
\author
**/
#ifndef IFleetForwardListBased_HPP
#define IFleetForwardListBased_HPP


#include <forward_list>
#include <irrlicht.h>
#include <fusion/maths/macros.hpp>
#include <fusion/logging/ILogger.hpp>
#include <array>
#include "../def.hpp"
#include "IFleet.hpp"
#include "../utils/TClock.hpp"


class CPlayer;
class CMissile;
class CBullet;
class IStaticBattery;
class CStaticBattery;

namespace audio {

class IMusicManager;
class ISoundManager;

}
class IParticleManager;
class IPlayableState;


//typedef std::list<CStaticBattery*> TTrackedBatteries;
//: public CIntrusiveList
class IFleetForwardListBased : public IFleet
{
private:

protected:
    typedef  std::forward_list< IStaticBattery*> TBatteryContainer;
    TBatteryContainer _activeBatteries;


    //typedef std::array<CIntrusiveList,EEntityType::Count> TDeadPool;
    typedef std::array< TBatteryContainer, EEntityType::Count> TDeadPool;
    static  TDeadPool _deadPool;



    IStaticBattery* getDeadButAllocatedBattery(const EEntityType::EId& type);


    void registerActiveBattery(IStaticBattery* battery);

    // \brief equivalent to killing an active battery. Will then add it to the cache
//    void cache_after(TBatteryContainer::iterator& it,IStaticBattery* battery);

    void cacheBattery(IStaticBattery* battery );
    void cacheBattery(const EEntityType::EId& type,IStaticBattery* battery );

//    bool operator()(IStaticBattery* battery);

    bool sporadicBatteryCheck( IStaticBattery* );
    bool regularBatteryUpdate(const TTimer::TimeType& elapsedTime , IStaticBattery* );


    // TODO flushCach

public:
    IFleetForwardListBased( IPlayableState* state );
    virtual ~IFleetForwardListBased();

    std::size_t getNumberOfCachedEntities(const EEntityType::EId&) const;

    void sporadicCheck();

    /** \brief
     *
     * \return Number of entities in this fleet
     *
     */
//    irr::u32 getSize() const { return _activeBatteries.size(); };


    virtual std::size_t getNumberOfActiveEntities() const;
//    irr::scene::IAnimatedMesh* getMesh(const char* filename);

//    static void clearDeadPool();
//    static void destroyAll();
//    void killAllAlive();


//    void killBattery( IStaticBattery* );
//    void killBattery( TBatteryContainer::iterator& it);

    virtual irr::scene::ISceneNode *getCachedFleetNode() ;
    virtual irr::scene::ISceneNode *getActiveFleetNode() ;


    /// TODO old
    IStaticBattery* getNearestEnemy(const TVector3df& position);

    // TODO passer un forward_list
    IStaticBattery* getNextTarget(const std::list<IStaticBattery*>& list);



    void update(const TTimer::TimeType&);


    // onFadingDisparition
    virtual void onTimeoutDisparition( IStaticBattery* ) = 0;

protected:
    virtual void onCollision(IStaticBattery* entity1,IStaticBattery* entity2) = 0;
    //TBulletList::iterator deleteBullet(CBullet&);


};

#endif // CENEMYFLEET_HPP
