/**
* \file options.h
*  \brief Option menu state
*
*   State called from menu, single and MultiPlayerState
*   Basée sur menu
**/

#ifndef OPTIONSTATE_HPP
#define OPTIONSTATE_HPP

#include "IGameState.hpp"
#include "../startup_config.hpp"
#include <irrlicht.h>
#include <boost/optional.hpp>
#include <array>
#include <map>



// Xeffects
class EffectHandler;

    class CBindFromEventGenerator;

DECLARE_GAME_STATE( COptionState )
//class COptionState : public IGameState<NStateId::Options,false>
    //"Options"
{

private:
    enum EOptionTab {
    Graphics,
    Controls,
    Sound,
    Gameplay,
    Count
    };


    INNER_STATE_DECLARATION( COptionState )


    SIngameConfig GameConfig;
//    irr::video::ITexture* Background;

public:

    /** \brief Copies current config
    **/
    COptionState(SIngameConfig const&);
    ~COptionState();

	bool load( CGameEngine*,irr::scene::ISceneManager*,irr::video::ITexture* );
	void unload();


//	bool isLoaded() const;
	bool OnEvent(const irr::SEvent&);

	void onUpdate(const TTimeUnit&);
	void Draw();


    std::wstring generateDebugData() const;



protected:
    boost::scoped_ptr<CBindFromEventGenerator> _descriptorGenerator;

    void setButtonValue(NPlayerInput::EId const& id);
    void createBindLine(NPlayerInput::EId const& id, irr::gui::IGUITab* tab);

    bool waitingNewBind() const {
        //return (_indexOfFocusedElement >= 0);
        //return (_translatedIdToUpdate);
        return (_descriptorGenerator != 0);
    }

// TODO unused !!
    bool processEventForSoundTab(const irr::SEvent::SGUIEvent& event);
    bool processEventForControlTab(const irr::SEvent::SGUIEvent& event);
    bool processEventForGraphicTab(const irr::SEvent::SGUIEvent& event);
    bool processEventForGameplayTab(const irr::SEvent::SGUIEvent& event);

    // "setup" plutot que "create"
    void createSoundTab(irr::gui::IGUITab* tab);
    void createControlTab(irr::gui::IGUITab* tab);
    void createGraphicTab(irr::gui::IGUITab* tab);
    void createGameplayTab(irr::gui::IGUITab* tab);


    void OnClickBack();
    bool OnClickSaveButton();

    void GUI_CreateMainMenu();              //!< Create main options menu

//reset ?
    void setToConfigValues();
    irr::core::stringw videoModeToText(const irr::core::dimension2du& dims,const irr::s32& depth);

//    typedef void (CBattery::*TUpdateFunction)(const float&);
    typedef bool (COptionState::*TProcessingFunction)(const irr::SEvent::SGUIEvent&);

    TProcessingFunction _processingFunctions[Count];

//!< Inform what tab we are on
//renommer en _tabControl
    //std::array<const wchar_t*, NPlayerInput::Count >;

    irr::core::rectf _pos;
    irr::gui::IGUITab* _overallTab;
    irr::gui::IGUITabControl* _tabControl;
    irr::gui::IGUIButton* _buttonToUpdate;


    // SIngameConfig
    //irr::SConf

    irr::gui::IGUIComboBox* _comboBox;



    // TODO informer l'utilisateur si l'application doit redémarrer ou pas.
    // On enregistre la version avant config,
    irr::SIrrlichtCreationParameters _bufferedParameters;
    // on recupere la version après et on compare les 2,
    // s'il y a des différences alors faut redémarrer le programme !

    //de  depart
    //bool _needRestart;
};

#endif // OPTIONSTATE_H
